local compost = CompostLib:GetInstance('compost-1')
local tablet = TabletLib:GetInstance('1.0')
local dewdrop = DewdropLib:GetInstance('1.0')
local metro = Metrognome:GetInstance('1')

AmmoFu = FuBarPlugin:GetInstance("1.2"):new({
	name          = AmmoFuLocals.NAME,
	description   = AmmoFuLocals.DESCRIPTION,
	version       = "1.1." .. string.sub("$Revision: 1551 $", 12, -3),
	releaseDate   = string.sub("$Date: 2006-05-06 23:08:11 -0500 (Sat, 06 May 2006) $", 8, 17),
	aceCompatible = 103,
	author        = "ckknight",
	email         = "ckknight@gmail.com",
	website       = "http://ckknight.wowinterface.com/",
	category      = "inventory",
	db            = AceDatabase:new("AmmoFuDB"),
	defaults      = {
		showPercent = false,
	},
	cmd           = AceChatCmd:new(AmmoFuLocals.COMMANDS, AmmoFuLocals.CMD_OPTIONS),
	loc           = AmmoFuLocals,
	hasIcon       = "Interface\\AddOns\\FuBar_AmmoFu\\arrow.tga",
	cannotDetachTooltip = TRUE,
})

function AmmoFu:IsShowingPercent()
	return self.data.showPercent
end

function AmmoFu:ToggleShowingPercent(loud)
	self.data.showPercent = not self.data.showPercent
	if loud then
		self.cmd:status(self.loc.ARGUMENT_PERCENT, self.data.showPercent and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	self:Update()
	return self.data.showPercent
end

function AmmoFu:Initialize()
	if not self.data.initialized then
		self.data.initialized = TRUE
		_,class = UnitClass("player")
		if class ~= "WARRIOR" and class ~= "ROGUE" and class ~= "HUNTER" or class ~= "WARLOCK" then
			self:Hide()
		end
	end
	self.timeSinceLastUpdate = 0
end

function AmmoFu:Enable()
	self:RegisterEvent("UNIT_MODEL_CHANGED", "OnUnitModelChanged")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "OnPlayerLeavingWorld")
	self:RegisterEvent("BAG_UPDATE", "OnBagUpdate")
	self:RegisterEvent("ITEM_LOCK_CHANGED", "OnItemLockChanged")
	metro:Register(self.name, self.OnUpdate, 1, self)
end

function AmmoFu:Disable()
	metro:Unregister(self.name)
end

function AmmoFu:Report()
	local report = compost:Acquire(
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_PERCENT,
			'val', self:IsShowingPercent() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		)
	)
	self.cmd:report(report)
	compost:Reclaim(report, 1)
end

function AmmoFu:MenuSettings(level, value)
	if level == 1 then
		dewdrop:AddLine(
			'text', self.loc.MENU_SHOW_AS_PERCENTAGE,
			'arg1', self,
			'func', "ToggleShowingPercent",
			'checked', self:IsShowingPercent()
		)
	end
end

function AmmoFu:OnPlayerEnteringWorld()
	self:RegisterEvent("BAG_UPDATE", "OnBagUpdate")
	self:RegisterEvent("ITEM_LOCK_CHANGED", "OnItemLockChanged")
end

function AmmoFu:OnPlayerLeavingWorld()
	self:UnregisterEvent("BAG_UPDATE")
	self:UnregisterEvent("ITEM_LOCK_CHANGED")
end

function AmmoFu:OnBagUpdate()
	self:Update()
end

function AmmoFu:OnItemLockChanged()
	metro:Start(self.name)
end

function AmmoFu:OnUnitModelChanged()
	self:Update()
end

function AmmoFu:OnClick()
	for i = 1, 4 do
		local link = GetInventoryItemLink("player", ContainerIDToInventoryID(i))
		if link ~= nil then
			local _,_,_,_,_,subtype = GetItemInfo(tonumber(string.gsub(link, "|cff%x%x%x%x%x%x|Hitem:(%d+):%d+:%d+:%d+|h.*", "%1") or 0))
			if subtype == self.loc.TEXT_SOUL_BAG or subtype == self.loc.TEXT_AMMO_POUCH or subtype == self.loc.TEXT_QUIVER then
				ToggleBag(i)
			end
		end
	end
end

function AmmoFu:OnUpdate(t)
	metro:Stop(self.name)
	self:Update()
end

function AmmoFu:UpdateText()
	local ammoSlotId = GetInventorySlotInfo("ammoSlot")
	local rangedSlotId = GetInventorySlotInfo("rangedSlot")

	local isThrown = FALSE
	local link = GetInventoryItemLink("player", rangedSlotId)
	if link ~= nil then
		local _,_,_,_,_,subtype = GetItemInfo(tonumber(string.gsub(link, "|cff%x%x%x%x%x%x|Hitem:(%d+):%d+:%d+:%d+|h.*", "%1") or 0))
		if GetInventoryItemQuality("player", rangedSlotId) ~= nil and subtype == self.loc.TEXT_THROWN then
			isThrown = TRUE
		end
	end
	
	if not isThrown and GetInventoryItemQuality("player", ammoSlotId) ~= nil then
		isAmmo = TRUE
	end
	
	local current = 0
	local total = 0
	local _,class = UnitClass("player")
	if class == "WARLOCK" then
		self:SetIcon("Interface\\AddOns\\" .. self.realName .. "\\shard.tga")
		for i = 0, 4 do
			if i >= 1 then
				local link = GetInventoryItemLink("player", ContainerIDToInventoryID(i))
				if link ~= nil then
					local itemId = tonumber(string.gsub(link, "|cff%x%x%x%x%x%x|Hitem:(%d+):%d+:%d+:%d+|h.*", "%1") or 0)
					local _,_,_,_,_,subtype = GetItemInfo(itemId)
					if subtype == self.loc.TEXT_SOUL_BAG then
						local size = GetContainerNumSlots(i)
						if size ~= nil and size > 0 then
							total = total + size
						end
					end
				end
			end
			for j = 1, GetContainerNumSlots(i) do
				local link = GetContainerItemLink(i, j)
				if link ~= nil and string.find(link, "|cff%x%x%x%x%x%x|Hitem:6265:%d+:%d+:%d+|h") then
					current = current + 1
				end
			end
		end
		if current > total then
			total = current
		end
		if total < 4 then
			total = 4
		end
	elseif isThrown then
		self:SetIcon("Interface\\AddOns\\" .. self.realName .. "\\knife.tga")
		current = GetInventoryItemCount("player", rangedSlotId)
		total = 100
	elseif isAmmo then
		self:SetIcon("Interface\\AddOns\\" .. self.realName .. "\\arrow.tga")
		current = GetInventoryItemCount("player", ammoSlotId)
		total = 0
		for i = 0, 4 do
			if i >= 1 then
				local link = GetInventoryItemLink("player", ContainerIDToInventoryID(i))
				if link ~= nil then
					local itemId = tonumber(string.gsub(link, "|cff%x%x%x%x%x%x|Hitem:(%d+):%d+:%d+:%d+|h.*", "%1") or 0)
					local _,_,_,_,_,subtype = GetItemInfo(itemId)
					if subtype == self.loc.TEXT_QUIVER or subtype == self.loc.TEXT_AMMO_POUCH then
						local size = GetContainerNumSlots(i)
						if size ~= nil and size > 0 then
							total = total + size * 200
						end
					end
				end
			end
		end
		if total == 0 then
			total = 200
		end
	else
		current = 0
		total = 0
	end
	
	local color
	if total == 0 then
		color = FuBarUtils.COLOR_HEX_RED
	else
		color = FuBarUtils.GetThresholdHexColor(current / total)
	end
	
	if self:IsShowingPercent() then
		self:SetText(format("|cff%s%d%%|r", color, current / total * 100))
	else
		self:SetText(format("|cff%s%d|r", color, current))
	end
end

function AmmoFu:UpdateTooltip()
	for i = 1, 4 do
		local link = GetInventoryItemLink("player", ContainerIDToInventoryID(i))
		if link ~= nil then
			local _,_,_,_,_,subtype = GetItemInfo(tonumber(string.gsub(link, "|cff%x%x%x%x%x%x|Hitem:(%d+):%d+:%d+:%d+|h.*", "%1") or 0))
			if subtype == self.loc.TEXT_SOUL_BAG then
				tablet:SetHint(self.loc.TEXT_HINT_SHARD)
				break
			elseif subtype == self.loc.TEXT_AMMO_POUCH or subtype == self.loc.TEXT_QUIVER then
				tablet:SetHint(self.loc.TEXT_HINT_AMMO)
				break
			end
		end
	end
end

AmmoFu:RegisterForLoad()
